@@ -1,11 +1,30 @@
+# The MIT License (MIT)
+#
 # Copyright (c) 2014 JohnyMoSwag <johnymoswag@gmail.com>
+#
+# Permission is hereby granted, free of charge, to any person obtaining a copy
+# of this software and associated documentation files (the "Software"), to deal
+# in the Software without restriction, including without limitation the rights
+# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
+# copies of the Software, and to permit persons to whom the Software is
+# furnished to do so, subject to the following conditions:
+#
+# The above copyright notice and this permission notice shall be included in
+# all copies or substantial portions of the Software.
+
+# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
+# SOFTWARE.
 
 import json
 import logging
 import os
 import re
 import shutil
-import stat
 import sys
 import urllib2
 from zipfile import ZipFile
@@ -16,6 +35,9 @@ from .utils import (ChDir, rsa_verify, _version_string_to_tuple,
                     _version_tuple_to_string, platform_)
 
 log = logging.getLogger(__name__)
+c = logging.StreamHandler()
+c.setLevel(logging.INFO)
+log.addHandler(c)
 
 
 class Client(object):
@@ -84,7 +106,7 @@ class Client(object):
         if patch_success:
             log.info('Successful patch update')
             return True
-
+        log.debug('Patch updated failed...')
         update_success = self._full_update(name, version)
         if update_success:
             return True
@@ -92,10 +114,30 @@ class Client(object):
             return False
 
     def install_restart(self):
+        """Proxy method for :meth: `_unzip_file`,
+        :meth: `_remove_version_number` & :meth: `_move_restart` Downloaded
+        updated will be unzipped, moved to the destination location and
+        restarted as updated application.
+        """
         self._unzip_file()
         self._remove_version_number()
         self._move_restart()
 
+    def install(self):
+        """Proxy method for :meth: `_unzip_file` &
+        :meth: `_remove_version_number`.  Downloaded update will be unzipped
+        ready to be moved to destination and the application restarted. This
+        call should be followed by :meth: `restart` to complete update.
+        """
+        self._unzip_file()
+        self._remove_version_number()
+
+    def restart(self):
+        """Proxy method for :meth: `_move_restart`.  Will move update to
+        destination and restart updated application.
+        """
+        self._move_restart()
+
     def _unzip_file(self):
         platform_name = self.name
         if sys.platform == 'win32':
@@ -104,6 +146,7 @@ class Client(object):
         filename = self._get_filename_with_extension(self.name,
                                                      highest_version)
         with ChDir(self.update_folder):
+            log.info('Extracting Update')
             z_file = ZipFile(filename)
             data = z_file.read(platform_name)
             with open(platform_name, 'wb') as f:
@@ -117,7 +160,8 @@ class Client(object):
         filename = self._get_filename_with_extension(self.name,
                                                      highest_version)
         with ChDir(self.update_folder):
-            self.fixed_filename = re.sub('-' + self.plat + '-[0-9]+\.[0-9]+\.[0-9]+.*', '',
+            self.fixed_filename = re.sub('-' + self.plat +
+                                         '-[0-9]+\.[0-9]+\.[0-9]+.*', '',
                                          filename)
 
     def _move_restart(self):
@@ -126,9 +170,8 @@ class Client(object):
         updated_app = os.path.join(self.update_folder, self.fixed_filename)
         if sys.platform != 'win32':
             shutil.move(updated_app, current_app)
-            os.chmod(current_app, stat.S_IXUSR)
-            print 'Restarting'
             os.chmod(current_app, 0755)
+            log.info('Restarting')
             os.execv(current_app, [current_app])
         else:
             current_app += '.exe'
@@ -166,22 +209,25 @@ start {} "{}" """.format(updated_app, current_app, fix, current_app))
                 False - Either failed to patch or no base binary to patch
 
         """
+        log.info('Starting patch update')
         try:
-            filename = self.json_data['y_data'][name][version][self.plat]['filename']
+            version_plat = self.json_data['y_data'][name][version][self.plat]
+            filename = version_plat['filename']
             highest_version = self._get_highest_version(name)
         except Exception as e:
             log.error(str(e), exc_info=True)
             log.warning('Supplied version number not found')
             return False
-        if _version_string_to_tuple(highest_version) <= _version_string_to_tuple(version):
-            log.info('{} already updated to the latest version'.format(name))
+        if _version_string_to_tuple(highest_version) <= \
+                _version_string_to_tuple(version):
+            log.debug('{} already updated to the latest version'.format(name))
             self.up_to_date = True
             return True
 
         # Just checking to see if the latest update is available to patch
         # If not we'll just do a full binary download
         if not os.path.exists(os.path.join(self.update_folder, filename)):
-            log.warning('{} got deleted. No base to start patching '
+            log.warning('{} got deleted. No base binary to start patching '
                         'form'.format(filename))
             return False
 
@@ -190,7 +236,6 @@ start {} "{}" """.format(updated_app, current_app, fix, current_app))
 
         # Returns True if everything went well
         result = p.start()
-        log.info('Was patch update successful? {}'.format(result))
         return result
 
     def _full_update(self, name, version):
@@ -209,23 +254,16 @@ start {} "{}" """.format(updated_app, current_app, fix, current_app))
                 False - Update Failed
         """
         # Takes resource name and current installed version number
-        #
-        # Does the actual heavy lifting.  Gets highest version
-        # number then compares with pass version arg.
         highest_version = self._get_highest_version(name)
-        if _version_string_to_tuple(highest_version) <= _version_string_to_tuple(version):
-            log.info('{} already updated to the latest version'.format(name))
-            self.up_to_date = True
-            return True
-
         url = self._get_file_url(name, highest_version)
         filename = self._get_filename_with_extension(name,
                                                      highest_version)
         log.info('Starting full update')
         with ChDir(self.update_folder):
             # _hash = self.json_data['y_data'][name][version]['md5']
-            _hash = self.json_data['y_data'][name][highest_version][self.plat]['md5']
-            log.info('Downloading update')
+            version_ = self.json_data['y_data'][name][highest_version]
+            _hash = version_[self.plat]['md5']
+            log.info('Downloading update...')
             fd = FileDownloader(filename, url, _hash)
             result = fd.download_verify_write()
             if result:
@@ -256,7 +294,7 @@ start {} "{}" """.format(updated_app, current_app, fix, current_app))
 
         if 'sig' in self.json_data.keys():
             self.sig = self.json_data['sig']
-            log.info('Deleting sig from update data')
+            log.debug('Deleting sig from update data')
             del self.json_data['sig']
             update_data = json.dumps(self.json_data, sort_keys=True)
             result = rsa_verify(update_data, self.sig, self.public_key)
@@ -275,7 +313,7 @@ start {} "{}" """.format(updated_app, current_app, fix, current_app))
             v = urllib2.urlopen(self.version_url)
             self.json_data = json.loads(v.read())
         except Exception as e:
-            log.error(str(e), exc_info=True)
+            log.error(str(e))
             self.json_data = None
 
     def _get_highest_version(self, name):
@@ -288,8 +326,10 @@ start {} "{}" """.format(updated_app, current_app, fix, current_app))
             (str) Highest version number
 
         """
-        highest_version = _version_tuple_to_string(sorted(map(_version_string_to_tuple,
-                                                   self.json_data['y_data'][name].keys()))[-1])
+        highest_version = _version_tuple_to_string(sorted(map(
+                                                   _version_string_to_tuple,
+                                                   self.json_data['y_data']
+                                                   [name].keys()))[-1])
         log.debug('Highest version: {}'.format(highest_version))
         return highest_version
 
@@ -320,7 +360,9 @@ start {} "{}" """.format(updated_app, current_app, fix, current_app))
             (str) Filename with extension
         """
         # Parses version file and returns filename
-        filename = self.json_data['y_data'][name][version][self.plat]['filename']
+        version_plat = self.json_data['y_data'][name][version][self.plat]
+        # Used fro pep 8 compliance
+        filename = version_plat['filename']
         log.debug("Filename: {}".format(filename))
         return filename
 
