@@ -1,22 +1,43 @@
+# The MIT License (MIT)
+#
 # Copyright (c) 2014 JohnyMoSwag <johnymoswag@gmail.com>
+#
+# Permission is hereby granted, free of charge, to any person obtaining a copy
+# of this software and associated documentation files (the "Software"), to deal
+# in the Software without restriction, including without limitation the rights
+# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
+# copies of the Software, and to permit persons to whom the Software is
+# furnished to do so, subject to the following conditions:
+#
+# The above copyright notice and this permission notice shall be included in
+# all copies or substantial portions of the Software.
+
+# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
+# SOFTWARE.
 
 import json
 import logging
+import multiprocessing
 import os
 import re
 import shutil
 
-try:
-    import bsdiff4
-    PATCH_SUPPORT = True
-except ImportError:
-    PATCH_SUPPORT = False
-
 from . import PackageHandlerError
 from .utils import (ChDir, ask_yes_no, _get_package_hashes,
                     _remove_hidden_stuff_mac, _version_string_to_tuple,
                     _version_tuple_to_string)
 
+try:
+    import bsdiff4
+except ImportError:
+    raise PackageHandlerError('You must have bsdiff4 installed.',
+                              expected=True)
+
 log = logging.getLogger(__name__)
 
 
@@ -44,11 +65,8 @@ class PackageHandler(object):
         """
         self.app_dir = obj.config.get('APP_DIR')
 
-        # ToDo Remove to enable patches once implementation
-        # is complete.
-        # self.patches = obj.config.get('UPDATE_PATCHES', False)
-        self.patches = True
-        if PATCH_SUPPORT and self.patches:
+        self.patches = obj.config.get('UPDATE_PATCHES', True)
+        if self.patches:
             log.debug('Looks like were ready to make some patches')
             self.patch_support = True
         else:
@@ -91,7 +109,7 @@ class PackageHandler(object):
             - Data dir
         Then inside the data folder it creates 3 more folders
             - New - for new updates that need to be signed
-            - Deploy - After signing, where everything is place ready for upload
+            - Deploy - All files ready to upload are placed here.
             - Files - All updates are placed here for future reference
         """
         # Creates build directories if not already
@@ -253,12 +271,15 @@ class PackageHandler(object):
                 info['patch_md5'] = patch_hash
                 info['patch_url'] = self.update_url + patch_name
 
-            log.debug('json version {}'.format(self.json_data['y_data'][name].get(version, None)))
+            log.debug('json version {}'.format(self.json_data['y_data']
+                      [name].get(version, None)))
             if self.json_data['y_data'][name].get(version, None) is None:
                 log.debug('Adding new version to file')
                 self.json_data['y_data'][name][version] = {}
-                if self.json_data['y_data'][name][version].get('platform', None) is None:
-                    self.json_data['y_data'][name][version][p['platform']] = info
+                if self.json_data['y_data'][name][version].get('platform',
+                                                               None) is None:
+                    name_ = self.json_data['y_data'][name]
+                    name_[version][p['platform']] = info
             else:
                 log.debug('Appending info data to version file')
                 self.json_data['y_data'][name][version][p['platform']] = info
@@ -270,6 +291,7 @@ class PackageHandler(object):
         # Process all packages in new folder and gets
         # url md5 and some outer info.
         log.info('Getting package list')
+        patch_manifest = []
         with ChDir(self.new_dir):
             packages = os.listdir(os.getcwd())
             for p in packages:
@@ -290,15 +312,31 @@ class PackageHandler(object):
                                 'filename': p,
                                 'md5': _hash,
                                 'platform': platform}
+                self.package_manifest.append(package_info)
                 path = self._check_make_patch(p_name, p_version, platform)
                 if self.patch_support and path is not None:
                     platform_patch_name = p_name + '-' + platform
-                    patch_name = self._make_patch(path[0], p,
-                                                  platform_patch_name, path[1])
-                    patch_hash = _get_package_hashes(patch_name)
-                    package_info['patch_name'] = patch_name
-                    package_info['patch_md5'] = patch_hash
-                self.package_manifest.append(package_info)
+                    patch_info = dict(src=path[0], dst=p,
+                                      patch_name=platform_patch_name,
+                                      patch_num=path[1], package=p)
+
+                    patch_manifest.append(patch_info)
+
+            cpu_count = multiprocessing.cpu_count() * 2
+            pool = multiprocessing.Pool(processes=cpu_count)
+            pool_output = pool.map(_make_patch, patch_manifest)
+
+            for i in pool_output:
+                for s in self.package_manifest:
+                    if i[0] == s['real_name']:
+                        s['patch_name'] = i[1]
+                        s['patch_md5'] = _get_package_hashes(i[1])
+                        break
+
+            with open('manifest.txt', 'w') as m:
+                m.write(json.dumps(self.package_manifest, indent=2,
+                        sort_keys=True))
+                m.write('\nDone!!')
 
     def _get_platform(self, name):
         platform_name = re.compile('[macnixw0-9]{3,5}').findall(name)[0]
@@ -357,28 +395,6 @@ class PackageHandler(object):
                 return None
             return src_file_path, len(_files) + 1
 
-    def _make_patch(self, src_path, dst_path, patch_name, patch_number):
-        """Makes patch
-
-        Args:
-            src_path (str): Path to src binary
-
-            dst_path (str): Path to dst binary
-
-            patch_name (str): Name to give created patch
-
-            patch_number (str): Number to append to patch name
-
-        Returns:
-            (str) Final patch name
-        """
-
-        patch_name = patch_name + '-' + str(patch_number)
-        log.info('Creating patch')
-        bsdiff4.file_diff(src_path, dst_path, patch_name)
-        log.info('Done creating patch')
-        return patch_name
-
     def _remove_ext(self, filename):
         """Removes file ext from filename.
 
@@ -416,7 +432,8 @@ class PackageHandler(object):
             v_n = re.compile('[0-9]+\.[0-9]+\.[0-9]+').findall(package_name)[0]
             return v_n
         except IndexError:
-            raise PackageHandlerError('Package {} not formatted correctly'.format(package_name),
+            raise PackageHandlerError('Package {} not formatted '
+                                      'correctly'.format(package_name),
                                       expected=True)
 
     def _remove_version_number(self, package_name):
@@ -432,3 +449,34 @@ class PackageHandler(object):
         # What, thought it did something else? lol j/k
         reg = '-[0-9]+\.[0-9]+\.[0-9]+'
         return re.sub(reg, '', package_name)
+
+
+def _make_patch(patch_info):
+        """Makes patch
+
+        Args:
+            src_path (str): Path to src binary
+
+            dst_path (str): Path to dst binary
+
+            patch_name (str): Name to give created patch
+
+            patch_number (str): Number to append to patch name
+
+        Returns:
+            (str) Final patch name
+        """
+        patch_name = patch_info['patch_name']
+        patch_number = patch_info['patch_num']
+        src_path = patch_info['src']
+        dst_path = patch_info['dst']
+        print "Patch name: {}".format(patch_name)
+        print 'Patch number: {}'.format(patch_number)
+        print 'Src: {}'.format(src_path)
+        print 'Dst: {}'.format(dst_path)
+
+        patch_name += '-' + str(patch_number)
+        log.info('Creating patch')
+        bsdiff4.file_diff(src_path, dst_path, patch_name)
+        log.info('Done creating patch')
+        return dst_path, patch_name
