@@ -1,9 +1,25 @@
+# The MIT License (MIT)
+#
 # Copyright (c) 2014 JohnyMoSwag <johnymoswag@gmail.com>
+#
+# Permission is hereby granted, free of charge, to any person obtaining a copy
+# of this software and associated documentation files (the "Software"), to deal
+# in the Software without restriction, including without limitation the rights
+# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
+# copies of the Software, and to permit persons to whom the Software is
+# furnished to do so, subject to the following conditions:
+#
+# The above copyright notice and this permission notice shall be included in
+# all copies or substantial portions of the Software.
+
+# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
+# SOFTWARE.
 
-try:
-    import bsdiff4
-except ImportError:
-    bsdiff4 = None
 import hashlib
 import logging
 import os
@@ -12,6 +28,11 @@ from .file_downloader import FileDownloader
 from .utils import (ChDir, _version_string_to_tuple,
                     _version_tuple_to_string, platform_)
 
+try:
+    import bsdiff4
+except ImportError:
+    bsdiff4 = None
+
 log = logging.getLogger(__name__)
 
 
@@ -144,9 +165,12 @@ class Patcher(object):
             info = {}
             v_num = _version_tuple_to_string(p)
             try:
-                info['patch_name'] = self.json_data['y_data'][name][v_num][self.plat]['patch_name']
-                info['patch_url'] = self.json_data['y_data'][name][v_num][self.plat]['patch_url']
-                info['patch_md5'] = self.json_data['y_data'][name][v_num][self.plat]['patch_md5']
+                temp_name = self.json_data['y_data'][name][v_num][self.plat]
+                info['patch_name'] = temp_name['patch_name']
+                temp_url = self.json_data['y_data'][name][v_num][self.plat]
+                info['patch_url'] = temp_url['patch_url']
+                temp_hash = self.json_data['y_data'][name][v_num][self.plat]
+                info['patch_md5'] = temp_hash['patch_md5']
                 self.patch_data.append(info)
             except KeyError:
                 log.error('Missing required patch meta-data')
@@ -186,7 +210,8 @@ class Patcher(object):
     def _write_update_to_disk(self):
         """Writes updated binary to disk"""
         log.info('Writing update to disk')
-        filename = self.json_data['y_data'][self.name][self.highest_version][self.plat]['filename']
+        temp_filename = self.json_data['y_data'][self.name]
+        filename = temp_filename[self.highest_version][self.plat]['filename']
         with ChDir(self.update_folder):
             with open(filename, 'wb') as f:
                 f.write(self.new_binary)
@@ -205,8 +230,10 @@ class Patcher(object):
                 hash = dict['hash']
         """
         info = {}
-        info['filename'] = self.json_data['y_data'][name][version][self.plat]['filename']
-        info['hash'] = self.json_data['y_data'][name][version][self.plat]['md5']
+        temp_filename = self.json_data['y_data'][name][version][self.plat]
+        info['filename'] = temp_filename['filename']
+        temp_hash = self.json_data['y_data'][name][version][self.plat]
+        info['hash'] = temp_hash['md5']
         return info
 
     def _get_package_hashes(self, package):
